package org.xbib.interlibrary.client.hbz.z;

import org.xbib.interlibrary.action.resolve.ResolveRequest;
import org.xbib.interlibrary.action.resolve.ResolveResponse;
import org.xbib.interlibrary.action.resolve.ResolveResponseBuilder;
import org.xbib.interlibrary.action.resolve.ResolveServiceResponseBuilder;
import org.xbib.interlibrary.api.action.Service;
import org.xbib.interlibrary.api.action.ServiceArguments;
import org.xbib.interlibrary.common.DefaultBibliographicDescription;
import org.xbib.interlibrary.z.AbstractZResolveService;
import org.xbib.marc.MarcField;
import org.xbib.marc.MarcRecord;

import java.util.Locale;

public class ZResolveService extends AbstractZResolveService {

    public ZResolveService(ServiceArguments arguments) {
        super(arguments);
    }

    @Override
    protected String supports(String source) {
        return source != null && (source.toLowerCase(Locale.ROOT).contains("lvi") ||
                source.toLowerCase(Locale.ROOT).contains("hbz") ||
                source.toLowerCase(Locale.ROOT).contains("zdb")
        )? source.toUpperCase(Locale.ROOT) : null;
    }

    @Override
    protected void resolve(ResolveRequest resolveRequest,
                           ResolveServiceResponseBuilder serviceResponseBuilder,
                           DefaultBibliographicDescription.Builder descriptionBuilder,
                           MarcRecord marcRecord) {
        for (MarcField marcField : marcRecord.getAll("001")) {
            String value = marcField.getValue();
            if (value != null) {
                descriptionBuilder.setSource("HBZ");
                descriptionBuilder.setSourceId(value.trim());
            }
        }
        for (MarcField marcField : marcRecord.getAll("035")) {
            String value = marcField.getFirstSubfieldValue("a");
            if (value != null && value.startsWith("(DE-599)ZDB")) {
                descriptionBuilder.setSource("ZDB");
                descriptionBuilder.setSourceId(value.substring(11));
            }
        }
    }

    @Override
    public int compareTo(Service<ResolveRequest, ResolveResponse, ResolveResponseBuilder> o) {
        return getName().compareTo(o.getName());
    }
}
