package org.xbib.interlibrary.client.hbz.z.legacy;

import org.xbib.datastructures.common.MultiMap;
import org.xbib.datastructures.tiny.TinyMap;
import org.xbib.interlibrary.action.avail.AvailRequest;
import org.xbib.interlibrary.action.avail.AvailResponse;
import org.xbib.interlibrary.action.avail.AvailResponseBuilder;
import org.xbib.interlibrary.action.avail.AvailServiceResponseBuilder;
import org.xbib.interlibrary.api.library.Library;
import org.xbib.interlibrary.api.library.LibraryService;
import org.xbib.interlibrary.api.action.Response;
import org.xbib.interlibrary.api.library.DefaultLibraryService;
import org.xbib.interlibrary.api.action.Service;
import org.xbib.interlibrary.api.action.ServiceArguments;
import org.xbib.interlibrary.api.library.Material;
import org.xbib.interlibrary.api.library.Mode;
import org.xbib.interlibrary.z.AbstractZAvailService;
import org.xbib.marc.MarcField;
import org.xbib.marc.MarcRecord;

import java.util.List;
import java.util.Locale;
import java.util.Map;
import static org.xbib.interlibrary.api.library.Material.ONLINE_RESOURCE;

public class ZLegacyAvailService extends AbstractZAvailService {

    private final Map<String, Mode> statuscodes;

    public ZLegacyAvailService(ServiceArguments arguments) {
        super(arguments);
        statuscodes = getModeMap("/org/xbib/interlibrary/client/hbz/z/legacy/status.json");
    }

    @Override
    protected Map<String, Mode> getStatusCodes() {
        return statuscodes;
    }

    @Override
    protected String supports(String source) {
        return source.toLowerCase(Locale.ROOT).contains("hbz") ? "HBZ" : null;
    }

    @Override
    protected void avail(AvailRequest availRequest,
                         AvailServiceResponseBuilder availResponse,
                         MultiMap<String, LibraryService> multiMap,
                         MarcRecord marcRecord) {
        boolean found = false;
        if (availRequest.isBibliographicDescriptionEnabled()) {
            availResponse.setBibliographicDescription(marcRecord);
            TinyMap.Builder<String, Object> builder = TinyMap.builder();
            DisplayHelper.display(builder, marcHelper, marcRecord, getName(), getName().toUpperCase(Locale.ROOT), null,
                    marcHelper.getRecordIdSupplier(marcRecord));
            availResponse.setDisplay(builder.build());
        }
        String sourceid = marcRecord.getFirst("001").getValue();
        List<Material> materials = marcHelper.extractMaterial(marcRecord);
        for (MarcField marcField : marcRecord.getAll("088")) {
            String sigel = marcField.getFirstSubfieldValue("a");
            DefaultLibraryService.Builder serviceBuilder = DefaultLibraryService.builder();
            Library library = configuration.getLibrary(sigel);
            if (library != null) {
                serviceBuilder.setISIL(library.getISIL());
                serviceBuilder.setName(library.getName());
                serviceBuilder.setDomainPriority(domainPriority(availRequest, library));
                serviceBuilder.setLibraryPriority(libraryPriority(availRequest, library));
                serviceBuilder.setMaterials(materials);
                serviceBuilder.setServicePriority(materials.contains(ONLINE_RESOURCE) ? 1 : 2);
                DefaultLibraryService.DefaultLocation location = new DefaultLibraryService.DefaultLocation();
                location.setCollection(marcField.getFirstSubfieldValue("b"));
                location.setCallnumber(marcField.getFirstSubfieldValue("c"));
                location.setDescription(marcField.getFirstSubfieldValue("d"));
                serviceBuilder.addLocation(location);
                String status = marcField.getFirstSubfieldValue("e");
                if (status != null) {
                    Mode mode= getStatusCodes().get(status);
                    if (mode != null) {
                        serviceBuilder.setMode(mode);
                    }
                }
                serviceBuilder.setSource(getName());
                serviceBuilder.setSourceId(sourceid);
                serviceBuilder.setId(library.getISIL() + "/" + sourceid);
                multiMap.put(library.getISIL(), serviceBuilder.build());
            }
            found = true;
        }
        if (found) {
            availResponse.setStatus(getName(), Response.Status.OK);
        } else {
            availResponse.setStatus(getName(), Response.Status.EMPTY_RESPONSE);
        }
    }

    @Override
    public int compareTo(Service<AvailRequest, AvailResponse, AvailResponseBuilder> o) {
        return getName().compareTo(o.getName());
    }
}
